#ifndef _LOADMD3_H_
#define _LOADMD3_H_

// polovica sirky/vysky diery po gulke - odtlacku
#define wDecalHalfSize 0.1f
// maximalny dostrel
#define wRange 100.0f

// hlavicka MD3 suboru
struct MD3header {
 unsigned char ident[4]; // identifikator=IDP3 (4x1B=4B)
 int version;            // verzia=MD3_VERSION (4B)
 unsigned char name[64]; // nazov MD3 suboru (null-terminated string, MAX_QPATH=64)
 int flags;              // ? (4B)
 int numFrames;          // pocet snimok - max. 1024 (4B)
 int numTags;            // pocet znaciek - max. 16 (4B)
 int numSurfaces;        // pocet sietovych modelov - max. 32 (4B)
 int numSkins;           // pocet povrchov (4B)
 int ofsFrames;          // zaciatok objektov Frames (4B)
 int ofsTags;            // zaciatok objektov Tags (4B)
 int ofsSurfaces;        // zaciatok objektov Surfaces (4B)
 int ofsEof;             // koniec hlavicky MD3 suboru (4B)
 // Frames
 // Tags
 // Surfaces
};

// Frame - snimka
struct MD3frame {
 float min[3];           // prvy roh ohranicujucej kocky (3x4B=12B)
 float max[3];           // druhy roh ohranicujucej kocky (3x4B=12B)
 float origin[3];        // lokalna poloha (3x4B=12B)
 float radius;           // polomer ohranicujucej gule (4B)
 unsigned char name[16]; // nazov snimky (null-terminated string, 16B)
};

// Tag - znacka - pouziva sa na usporiadanie sietovych modelov,
// standardne znacky su: TAG_TORSO, TAG_LEGS, TAG_HEAD, TAG_WEAPON
struct MD3tag {
 unsigned char name[64]; // nazov znacky (null-terminated string, MAX_QPATH=64)
 float origin[3];        // poloha znacky (3x4B=12B)
 float axis[3][3];       // orientacia znacky (3x3x4B=36B)
};

// Surface header - hlavicka sietoveho modelu
struct MD3surface_header {
 unsigned char ident[4]; // identifikator=IDP3 (4x1B=4B)
 unsigned char name[64]; // nazov sietoveho modelu (null-terminated string, MAX_QPATH=64)
 int flags;              // ? (4B)
 int numFrames;          // pocet snimok = pocet snimok v MD3header (4B)
 int numShaders;         // pocet shaderov - max. 256 (4B)
 int numVerts;           // pocet vrcholov - max. 4096 (4B)
 int numTriangles;       // pocet trojuholnikov - max. 8192 (4B)
 int ofsTriangles;       // zaciatok objektov Triangles (4B)
 int ofsShaders;         // zaciatok objektov Shaders (4B)
 int ofsSt;              // zaciatok objektov St - texture coordinates (4B)
 int ofsXyznormal;       // zaciatok objektov Vertex (4B)
 int ofsEnd;             // koniec hlavicky sietoveho modelu (4B)
 // Shaders
 // Triangles
 // St
 // Vertex (pocet vrcholov = numFrames x numVerts, pre kazdu snimku existuje 1 sada vrcholov)
};

// Shaders - shadery
struct MD3shader {
 unsigned char name[64]; // nazov shadera (null-terminated string, MAX_QPATH=64)
 int index;              // cislo shadera (4B)
};

// Triangles - odkazy na vrcholy trojuholnikov
struct MD3triangle {
 int indexes[3]; // odkazy na 3 vrcholy (3x4B=12B)
};

// St - texturovacie suradnice
struct MD3st {
 float st[2]; // ST (UV) texturovacie suradnice (2x4B=12B)
};

// Vertex - vrchol
struct MD3vertex {
 signed short int v[3]; // XYZ suradnice vrcholu (3x2B=6B)
 unsigned char n[2];    // zakodovany normalovy vektor (2x1B=2B)
};
// normaly su zakodovane: 15  14  13  12  11  10  9  8 | 7  6  5  4  3  2  1  0
//                                    latitude - sirka | longitude - dlzka
// kodovanie:
//  lng <- atan2 ( y / x) * 255 / (2 * pi)
//  lat <- acos ( z ) * 255 / (2 * pi)
//  lng <- lower 8 bits of lng
//  lat <- lower 8 bits of lat
//  normal <- (lat shift-left 8) binary-or (lng)
// specialne pripady:
//  up vektor - normal <- 0
//  down vektor - lat <- 0
//                lng <- 128
//                normal <- (lat shift-left 8) binary-or (lng) = 32768
// dekodovanie:
//  lat <- ((normal shift-right 8) binary-and 255) * (2 * pi ) / 255
//  lng <- (normal binary-and 255) * (2 * pi) / 255
//  x <- cos ( lat ) * sin ( lng )
//  y <- sin ( lat ) * sin ( lng )
//  z <- cos ( lng )

// MD3 sietovy model
struct MD3surface {
 MD3surface_header header; // hlavicka sietoveho modelu
 MD3shader *shaders;       // shadery
 MD3triangle *triangles;   // odkazy na vrcholy trojuholnikov
 MD3st *texcoords;         // texturovacie suradnice
 MD3vertex *vertices;      // vrcholy

 vec3 *tris_v;             // pole trojuholnikov
 vec2 *tris_t;             // pole texturovacich suradnic
};

// animacia
struct MD3anim {
 int firstF,numF,loopingF; // prva snimka, pocet snimok, pocet opakujucich sa snimok
 int fps;                  // pocet snimok za sekundu
};

// diera po gulke - odtlacok (decal), alebo cast odtlacku
struct wDecal {
 wDecal *next; // nasledujuci odtlacok

 vec3 v[9];    // vrcholy - orezanim 3-uholnika pomocou 6-tich orezavacich
               // rovin moze vzniknut max. 9-uholnik => odtlacok, alebo cast
               // odtlacku ma max. 9 vrcholov
 vec3 n;       // normala
 int idn;      // identifikator zavisly na normale
 int vnum;     // pocet vrcholov
 float alpha;  // priesvitnost

 //vec3 abcd[8]; // 8 vrcholov 3D hranola

 vec2 t[9];    // texturovacie suradnice
};

// trieda premennych a funkcii (objektov) pre nacitanie MD3 suborov
class CLoadMD3 {
public:
 CLoadMD3();  // konstruktor
 ~CLoadMD3(); // destruktor

 // nacitanie obsahu MD3 suboru
 int load(const char *FNAME);

 // vykreslenie 1 konkretnej snimky
 void drawModel1F(int AF);

 // vykreslenie MD3 modelu
 void draw(vec3 EP,vec3 TP,vec3 RP,GLubyte F);

 // vykreslenie dier po gulkach
 void drawDecals(void);
private:
 FILE *fp; // subor
 int fr;

 MD3header header;     // hlavicka MD3 suboru
 MD3frame *frames;     // snimky
 MD3tag *tags;         // znacky
 MD3surface *surfaces; // sietove modely
 CLoadMD3 **links;     // prepojenia

 float nxyz[256][256][3]; // tabulka predpripravenych normal

 int actframe,nextframe; // aktualna snimka, nasledujuca snimka
 float ipol;             // interpolacny cas <0.0;1.0>

 int stopaction;

 int actV,ofsV,ipos; // pomocne premenne

 vec3 frontV,rightV,upV; // vektory smerujuce dopredu, doprava a hore
 vec3 pV;                // pomocny vektor
 float ip0,ip1;          // pomocne premenne
 vec3 ipoint,inormal;    // priesecnik luca a trojuholnika, normala v priesecniku
 vec4 cplane[6];         // 6 orezavacich rovin, ktore su definovane stenami 3D hranola
 wDecal *firstD,*lastD,*curD,*cD; // prvy, posledny, aktualny odtlacok (casti odtlacku)
 wDecal *lastG;                   // oznacenie casti posledneho pridaneho odtlacku

 float texU,texV;   // pre nahodny vyber 1 zo 4 textur
 float angR,angU;   // uhly pre otocenie vrcholov do spolocnej roviny
 int ind;           // indikator najdenia spolocnej hrany (vrcholu)
 vec3 u1,u2,u3,u4;  // pomocne vektory
 float d1,d2,d3,d4; // pomocne premenne

 int addDecal(vec4 *P);    // pridanie odtlacku (vsetkych casti odtlacku)
 void delDecal(wDecal *P); // vymazanie odtlacku, alebo casti odtlacku
 void delAllDecals(void);  // zmazanie vsetkych odtlackov (vsetkych casti)
};

#endif
