#ifndef _CLOTH_H_
#define _CLOTH_H_

// preddefinovana hmotnost castice
#define PMASS 0.05f

// maximalna hodnota DT pre realnost simulacie, cim mensie DT tym lepsie,
// ak bude DT prilis velke, simulaciu je potrebne zopakovat viackrat
#define MAXDT 0.002f

// tuhost pruziny
#define SPRING_STIFFNESS 3000.0f

// trenie pruziny
#define SPRING_FRICTION 0.1f

// deklaracia triedy (nasleduje jej definicia)
class rigid_particle;

// obmedzenie pohybu castice
struct particle_constraint {
 rigid_particle *p; // "susedna" castica, s ktorou je aktualna castica zviazana silami
 float rest_dist;   // pociatocna vzdialenost (vzdialenost v pokoji) od susednej castice
};

// castica s nenulovou hmotnostou ******************************************
//  dolezite vzorce:
//   v=s/t s=v*t v=a*t [m/s, m, s]
//   a=v/t a=F/m       [m/s^2, m/s, s, N, kg]
//   F=m*a             [N, kg, m/s^2]
//   G=m*g
//   F=-k*d            k ... tuhost pruziny, d ... vzdialenost od uchytu
class rigid_particle {
private:
 float mass; // hmotnost castice

 vec3 pv; // pomocny vektor
public:
 vec3 pos,vel; // poloha a rychlost castice
 vec3 n;       // normala
 vec2 uv;      // texturovacie suradnice
 vec3 force;   // sila posobiaca na casticu

 float can_move; // castica sa moze/nemoze pohybovat

 /* vnutorne castice platna maju vazby na 1, 3, alebo 4 susedov,
    tieto vzajomne vazby obmedzuju pohyb castic
     .  .--.    .   . 0 .
       /|\            1
      / | \       3 2
     .  .  .    .   .   .
 */
 particle_constraint constraints[4];

 // konstruktor
 rigid_particle() { mass=PMASS; vel.set(0.0f,0.0f,0.0f); force.set(0.0f,0.0f,0.0f); can_move=1.0f; }

 // pridanie sily posobiacej na casticu
 void addForce(vec3 F) { force+=F; }

 // Eulerova simulacia fyzikalneho deja
 // zrychlenie castice = posobiaca sila / hmotnost castice
 //   rychlost castice = zrychlenie castice * cas
 //     poloha castice = rychlost castice * cas
 void simulate(float DT) {
  vel+=(force/mass)*DT*can_move; // v=a*t
  pos+=vel*DT;                   // s=v*t
 }
};

// trieda premennych a funkcii (objektov) pre pracu s platnom **************
class CCloth {
private:
 float width,height;       // sirka/vyska platna
 int width_num,height_num; // pocet castic v smere sirky/vysky platna

 vector<rigid_particle> particles; // zoznam castic
 int particles_num;                // pocet castic

 vector<_tri> cloth_tris; // zoznam trojuholnikov

 vec3 pv,vA,vB,v0,v1,v2; // pomocne vektory
 vec2 dv;                // pomocny vektor

 int iterations_num; // pocet opakovani simulacie
 float dt;           // rozdiel casu v ms

 float cur_dist; // aktualna vzdialenost dvoch castic

 int p0,p1,q; // pomocne premenne
 float f,g;   // pomocna premenna

 vec3 Gv,Wv,Fv; // vektory posobenia sil gravitacie, vetra a gulky
 vec3 Cv;       // pomocny kolizny vektor

 int col_id; // castica, ktora kolidovala s gulkou
public:
 CCloth();  // konstruktor
 ~CCloth(); // destruktor

 // inicializacia platna
 int init(void);

 // simulacia fyzikalneho deja
 void simulate(float DT,int CID);

 // vykreslenie platna
 void draw(vec3 EP,vec3 TP,GLubyte F);
};

#endif
